#include <Arduino.h>

#define HALL_SENSOR             A0

//Arduino ATmega328p and ATmega2560
#define SENSOR_POWER_SUPPLY     5 //VDC
#define ANALOG_RANGE            1023

//WCS1700 datasheet – https://html.alldatasheet.com/html-pdf/714468/WINSON/WCS1700/660/3/WCS1700.html
#define SENSOR_RESOLUTION       30 //mV
#define AVG_SAMPLES             10

const uint16_t voltageRange = SENSOR_POWER_SUPPLY * 1000;
uint16_t calcArray[AVG_SAMPLES];

uint16_t calibrationConstant = 0;

uint32_t getAverage(uint16_t *arrayPointer){
  uint32_t sum = 0;
  for(uint8_t i = 0; i < AVG_SAMPLES; i++){
    sum += arrayPointer[i];
  }
  return sum/AVG_SAMPLES;
}

void readCurrent(){
  uint16_t mapValue = 0;
  for(uint8_t i = 0; i < AVG_SAMPLES; i++){
    mapValue = map(analogRead(HALL_SENSOR), 0, ANALOG_RANGE, 0, voltageRange);
    if(mapValue > calibrationConstant){
      calcArray[i] = mapValue - calibrationConstant;
    }
    else{
      calcArray[i] = 0;
    }
    delay(20);
  }
}

void setup() {
  Serial.begin(9600, SERIAL_8N1);

  pinMode(HALL_SENSOR, INPUT);

  Serial.println("Calibrating, make sure that there is no current flow through wire.");
  delay(2500);
  for(uint8_t i = 0; i < AVG_SAMPLES; i++){
    calcArray[i] = map(analogRead(HALL_SENSOR), 0, ANALOG_RANGE, 0, voltageRange);
    Serial.print(".");
    delay(250);
  }
  Serial.println();
  calibrationConstant = getAverage(calcArray);
  Serial.print("Calibration value: "); Serial.println(calibrationConstant);
  delay(2500);
}

void loop() {
  readCurrent();
  float current = (float)getAverage(calcArray) / SENSOR_RESOLUTION;
  Serial.print("Actual current flow: "); Serial.print(current); Serial.println(" A");
  delay(1000);
}
